#!/usr/bin/env python
# PYTHON_ARGCOMPLETE_OK

"""
This is a skeleton file that can serve as a starting point for a Python
console script. To run this script uncomment the following lines in the
``[options.entry_points]`` section in ``setup.cfg``::

    console_scripts =
         fibonacci = giger.skeleton:run

Then run ``pip install .`` (or ``pip install -e .`` for editable mode)
which will install the command ``fibonacci`` inside your current environment.

Besides console scripts, the header (i.e. until ``_logger``...) of this file can
also be used as template for Python modules.

Note:
    This file can be renamed depending on your needs or safely removed if not needed.

References:
    - https://setuptools.pypa.io/en/latest/userguide/entry_point.html
    - https://pip.pypa.io/en/stable/reference/pip_install
"""

import argparse
import logging
import sys
import warnings

import argcomplete

from giger import __version__

from .commands.image import ImageCommand
from .commands.prompt import PromptCommand
from .commands.roop import RoopCommand
from .commands.template import TemplateCommand

warnings.filterwarnings("ignore", category=FutureWarning)


__author__ = "Sebastian Krüger"
__copyright__ = "Sebastian Krüger"
__license__ = "MIT"

_logger = logging.getLogger(__name__)


# ---- CLI ----
# The functions defined in this section are wrappers around the main Python
# API allowing them to be called directly from the terminal as a CLI
# executable/script.


class CLI:
    def parse_args(self, args):
        """
        Parse command line parameters
        """

        parser = argparse.ArgumentParser(
            prog="giger", description="Tools for Stable Diffusion"
        )

        parser.add_argument(
            "--version",
            action="version",
            version=f"giger {__version__}",
        )
        parser.add_argument(
            "-v",
            "--verbose",
            dest="loglevel",
            help="set loglevel to INFO",
            action="store_const",
            const=logging.INFO,
        )
        parser.add_argument(
            "-vv",
            "--very-verbose",
            dest="loglevel",
            help="set loglevel to DEBUG",
            action="store_const",
            const=logging.DEBUG,
        )

        subparsers = parser.add_subparsers(dest="command", required=True)

        self.template = TemplateCommand(subparsers)
        self.prompt = PromptCommand(subparsers)
        self.image = ImageCommand(subparsers)
        self.roop = RoopCommand(subparsers)

        argcomplete.autocomplete(parser)

        return parser.parse_args(args)

    def setup_logging(self, loglevel):
        """
        Setup basic logging

        Args:
            loglevel (int): minimum loglevel for emitting messages
        """
        logformat = "[%(asctime)s] %(levelname)s:%(name)s:%(message)s"
        logging.basicConfig(
            level=loglevel,
            stream=sys.stdout,
            format=logformat,
            datefmt="%Y-%m-%d %H:%M:%S",
        )

    def run(self, args):
        """
        Wrapper allowing :func:`fib` to be called with string arguments in a CLI fashion
        """
        args = self.parse_args(args)
        self.setup_logging(args.loglevel)
        if args.command == "template":
            self.template.run(args)
        elif args.command == "prompt":
            self.prompt.run(args)
        elif args.command == "image":
            self.image.run(args)
        elif args.command == "roop":
            self.roop.run(args)


def run():
    """
    Calls :func:`main` passing the CLI arguments extracted from :obj:`sys.argv`
    """
    CLI().run(sys.argv[1:])


if __name__ == "__main__":
    run()
